/* Useful functions for drawing Physics 102 figures.
 *
 * Copyright (C) 2008-2009 W. Trevor King <wking@drexel.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

import geometry;
import Mechanics;


// ---------------------- Charges -------------------------

// charged particle
struct Charge {
  pair center;
  real q;
  real radius;
  pen outline;
  pen fill;
  Label L;
  
  void operator init(pair center=(0,0), real q=1, real radius=2mm, pen outline=currentpen, pen fill=red, Label L="") {
    this.center = center;
    this.q = q;
    this.radius = radius;
    this.outline = outline;
    this.fill = fill;
    this.L = L;
  }
  
  void draw(picture pic=currentpicture) {
    picture picF;
    picture picL;
    label(picL, L);
    pair pLabelSize = 1.2 * (max(picL)-min(picL));
    path c = scale(radius)*unitcircle;
    filldraw(picF, c, fill, outline);
    label(pic = picF,
	  L = L,
	  position = (0,-(radius+pLabelSize.y/2)));
    add(pic, picF, center);
  }
}

// positive charge
Charge pCharge(pair center=(0,0), real q=1, real radius=2mm, pen outline=currentpen, Label L="")
{
  Charge c = Charge(center=center, q=q, radius=radius, outline=outline, L=L, fill=red);
  return c;
}

// negative charge
Charge nCharge(pair center=(0,0), real q=-1, real radius=2mm, pen outline=currentpen, Label L="")
{
  Charge c = Charge(center=center, q=q, radius=radius, outline=outline, L=L, fill=blue);
  return c;
}

// neutral charge
Charge neutralCharge(pair center=(0,0), real radius=2mm, pen outline=currentpen, Label L="")
{
  Charge c = Charge(center=center, q=0, radius=radius, outline=outline, L=L, fill=grey);
  return c;
}

// auto-signed charge
Charge aCharge(pair center=(0,0), real q=1, real radius=2mm, pen outline=currentpen, Label L="")
{
  Charge c;
  if (q > 0) {
    c = pCharge(center, q, radius, outline, L);
  } else if (q < 0) {
    c = nCharge(center, q, radius, outline, L);
  } else {
    c = neutralCharge(center, radius, outline, L);
  }
  return c;
}

// ---------------------- Vectors -------------------------

pen EFieldPen = rgb(1,0.5,0.2);    // orange
pen BFieldPen = rgb(0.1,1,0.2);    // green
pen CurrentPen = rgb(0.8,0.1,0.8); // purple

// electric field
Vector EField(pair center=(0,0), real mag=5mm, real dir=0, real phi=0, Label L="")
{
  Vector v = Vector(center=center, mag=mag, dir=dir, phi=phi, L=L, outline=EFieldPen);
  return v;
}

// magnetic field
Vector BField(pair center=(0,0), real mag=5mm, real dir=0, real phi=0, Label L="")
{
  Vector v = Vector(center=center, mag=mag, dir=dir, phi=phi, L=L, outline=BFieldPen);
  return v;
}

Vector Current(pair center=(0,0), real mag=5mm, real dir=0, real phi=0, Label L="")
{
  Vector v = Vector(center=center, mag=mag, dir=dir, phi=phi, L=L, outline=CurrentPen);
  return v;
}

// ---------------------- Forces -------------------------

// Force of a on b
Vector CoulombForce(Charge a, Charge b, Label L="", real scale=1mm, real unit=1mm)
{
  pair r = b.center - a.center;
  real mag, dir;
  mag = ((a.q*b.q)*(scale/length(r))^2)*unit;
  dir = degrees(r);
  Vector v = Force(center=b.center, mag=mag, dir=dir, L=L);
  return v;
}

void CoulombForces(Charge c[], real scale=1mm, real unit=1mm)
{
  Vector F;
  string s;
  for (int i=0; i<c.length; i+=1) {
    for(int j=0; j<c.length; j+=1) {
      if (i==j) continue;
      s = "$F_{" + format("%d", i+1) + "," + format("%d", j+1) + "}$";
      F = CoulombForce(c[i], c[j], L=s, scale=scale, unit=unit);
      F.draw();
    }
  }
}


